#include <stdio.h>
#include <string.h>
#include <math.h>

int batoi(char*);

int main(int argc,char *argv[])
{
	char* inname;
	char* outname;
	FILE* infile;
	FILE* outfile;
	int   translations=0;
	char  chr;
	char  binstr[1025];
	int   i;
	char  size=1;
	int   line=1;
	int   start=1;
	int   end=0;
	char  printstr[5];
	int   digits;
	if(argc<3 || (*(argv[1])=='-' && argc<4))
	{
		printf("TBTH: Converts binary numbers in TIGCC source files to hexadecimal ones.\n"
		       " Syntax: tbth [-{x|X}] infile outfile [startline [endline]]\n"
			   "  -x: use lowercase hexadecimal letters\n"
			   "  -X: use uppercase hexadecimal letters (default)\n"
			   "  infile: input file (c file)\n"
			   "  outfile: output file (c file). Must be different than infile.\n"
			   "  startline: line to start conversion. Must be supplied to use endline.\n"
			   "  endline: line to end conversion\n"
			   "\n"
			   "Programmed by Jonathan Long\n");
		return 0;
	}
	if(*(argv[1])=='-')
	{
		if(*(argv[1]+1)=='x')
			size=0;
		else if(*(argv[1]+1)=='X')
			size=1;
		else
			fprintf(stderr,"ERROR: Invalid option: %c\n",*(argv[1]+1));
		inname=argv[2];
		outname=argv[3];
		if(argc>=5)
			start=atoi(argv[4]);
		if(argc==6)
			end=atoi(argv[5]);
	}
	else
	{
		inname=argv[1];
		outname=argv[2];
		if(argc>=4)
			start=atoi(argv[3]);
		if(argc==5)
			end=atoi(argv[4]);
	}
	if(!strcmp(inname,outname))
	{
		fprintf(stderr,"ERROR: infile must be different than outfile\n");
		return 1;
	}
	if((infile=fopen(inname,"rb"))==NULL)
	{
		fprintf(stderr,"ERROR: cannot open %s\n",inname);
		return 1;
	}
	if((outfile=fopen(outname,"wb"))==NULL)
	{
		fprintf(stderr,"ERROR: cannot open or create %s\n",outname);
		return 1;
	}
	for(chr=getc(infile);chr!=EOF;chr=getc(infile))
	{
		putc(chr,outfile);
		if(chr=='0' && line>=start && (end ? line<=end : 1))
		{
			if((chr=getc(infile))=='b')
			{
				if((chr=getc(infile)), chr=='0' || chr=='1') 
				{
					putc('x',outfile);
					memset(binstr,0,1025);
					chr=getc(infile);
					for(i=0;chr=='0' || chr=='1';chr=getc(infile))
					{
						binstr[i]=chr;
						i++;
					}
					digits=(strlen(binstr)/4);
					
					if(size)
						sprintf(printstr,"%%.%dX",digits);
					else
						sprintf(printstr,"%%.%dx",digits);
					fprintf(outfile,printstr,batoi(binstr));
					fseek(infile,-1,SEEK_CUR);
					translations++;
					printf("Line %i: %s changed to hexidecimal\n",line,binstr);
				}
				else
					fseek(infile,-2,SEEK_CUR);
			}
			else
				fseek(infile,-1,SEEK_CUR);
		}
		else if(chr==13)
		{
			if((chr=getc(infile))==10)
				line++;
			fseek(infile,-1,SEEK_CUR);
		}
	}
	printf("File %s has been processed.\n" \
		   "%d binary numbers have been translated into hexadecimal numbers.\n" \
		   "The output has been written to file %s.\n",inname,translations,outname);
	fclose(outfile);
	fclose(infile);
	return 0;
}

int batoi(char* ba)
{
	int pos;
	int num=0;
	char value;
	for(pos=0;pos<strlen(ba);pos++)
	{
		if(*(ba+(strlen(ba)-1-pos))=='1')
			value=1;
		else
			value=0;
		num+=(int)pow(2.,(float)pos)*value;
	}
	return num;
}